@@ -12,14 +12,19 @@ import cPickle as pickle
 from cStringIO import StringIO
 from string import lowercase
 
+# Work around for some misguided Python packages that add iso-8859-1
+# accented characters to string.lowercase.
+lowercase = lowercase[:26]
+
 __version__ = '0.09 (Mailman edition)'
 VERSION = __version__
 CACHESIZE = 100    # Number of slots in the cache
 
+from Mailman import mm_cfg
 from Mailman import Errors
 from Mailman.Mailbox import ArchiverMailbox
 from Mailman.Logging.Syslog import syslog
-from Mailman.i18n import _
+from Mailman.i18n import _, C_
 
 # True/False
 try:
@@ -230,21 +235,30 @@ class Article:
         self.body = s.readlines()
 
     def _set_date(self, message):
-        def floatdate(header):
-            missing = []
-            datestr = message.get(header, missing)
-            if datestr is missing:
+        def floatdate(datestr):
+            if not datestr:
                 return None
             date = parsedate_tz(datestr)
             try:
-                return mktime_tz(date)
+                date = mktime_tz(date)
+                if (date < 0 or
+                    date - time.time() >
+                        mm_cfg.ARCHIVER_ALLOWABLE_SANE_DATE_SKEW
+                   ):
+                    return None
+                return date
             except (TypeError, ValueError, OverflowError):
                 return None
-        date = floatdate('date')
+        date = floatdate(message.get('date'))
         if date is None:
-            date = floatdate('x-list-received-date')
+            date = floatdate(message.get('x-list-received-date'))
+        if date is None:
+            rec_re = re.compile(r'^.*;\s*', re.DOTALL)
+            date = floatdate(rec_re.sub('', message.get('received', '')))
+        if date is None:
+            date = floatdate(re.sub(r'From \s*\S+\s+', '',
+                                    message.get_unixfrom() or '' ))
         if date is None:
-            # What's left to try?
             date = self._last_article_time + 1
         self._last_article_time = date
         self.date = '%011i' % date
@@ -285,7 +299,7 @@ class T:
             if errno != 2:
                 raise os.error, errdata
             else:
-                self.message(_('Creating archive directory ') + self.basedir)
+                self.message(C_('Creating archive directory ') + self.basedir)
                 omask = os.umask(0)
                 try:
                     os.mkdir(self.basedir, self.DIRMODE)
@@ -297,7 +311,7 @@ class T:
             if not reload:
                 raise IOError
             f = open(os.path.join(self.basedir, 'pipermail.pck'), 'r')
-            self.message(_('Reloading pickled archive state'))
+            self.message(C_('Reloading pickled archive state'))
             d = pickle.load(f)
             f.close()
             for key, value in d.items():
@@ -324,7 +338,7 @@ class T:
         self.update_TOC = 0
         self.write_TOC()
         # Save the collective state
-        self.message(_('Pickling archive state into ')
+        self.message(C_('Pickling archive state into ')
                      + os.path.join(self.basedir, 'pipermail.pck'))
         self.database.close()
         del self.database
@@ -436,7 +450,7 @@ class T:
     # dirtied or not.
     def update_archive(self, archive):
         self.archive = archive
-        self.message(_("Updating index files for archive [%(archive)s]"))
+        self.message(C_("Updating index files for archive [%(archive)s]"))
         arcdir = os.path.join(self.basedir, archive)
         self.__set_parameters(archive)
 
@@ -469,7 +483,7 @@ class T:
         self._restore_stdout()
 
     def _update_thread_index(self, archive, arcdir):
-        self.message(_("  Thread"))
+        self.message(C_("  Thread"))
         self._open_index_file_as_stdout(arcdir, "thread")
         self.type = 'Thread'
         self.write_index_header()
@@ -580,7 +594,7 @@ class T:
                 # It was an unparseable message
                 continue
             msgid = m.get('message-id', 'n/a')
-            self.message(_('#%(counter)05d %(msgid)s'))
+            self.message(C_('#%(counter)05d %(msgid)s'))
             a = self._makeArticle(m, self.sequence)
             self.sequence += 1
             self.add_article(a)
